;;;
;;; $Id$
;;; 
;;; File:		exmh-addr-completion.el
;;; Date:		Wed Apr 23 00:25:35 2008
;;; Author:		Alexander Zangerl (az)
;;; 
;;; Abstract:
;;;	access the exmh address database from emacs
;;;
;;; Copyright 2008 Alexander Zangerl
;;; Released under the GPL Version 1, http://www.gnu.org/copyleft/gpl.html

;; let's make the exmh address database useful!
(defvar exmh-addresses-lastread nil 
  "Time of last import of the exmh-addresses.")
(defvar exmh-addresses nil 
  "This holds the alist of addresses from the exmh address database.")

(defun read-exmh-addresses ( &optional force)
  "Reads the exmh address database and saves the addresses in exmh-addresses.
If the address db file is newer than exmh-addresses, no reading is performed (unless
the force argument is present)."
  (let ((abuf) (filetime) (vartime))
    (setq filetime (nth 5 (file-attributes "~/.exmh/exmh_addrs")))

    (if (or (boundp 'force)
	    (boundp 'exmh-addresses-lastread)
	    (boundp 'exmh-addresses))
	(setq exmh-addresses-lastread '(0 0)))
    (setq vartime exmh-addresses-lastread)

    (if (or (< (car vartime) (car filetime)) 
	    (and (= (car vartime) (car filetime)) 
		 (< (nth 1 vartime) (nth 1 filetime))))
	(save-excursion
	  (setq abuf (generate-new-buffer "exmhaddr"))
	  (set-buffer abuf)
	  (insert-file-contents "~/.exmh/exmh_addrs")
	  
	  (beginning-of-buffer)
	  (setq exmh-addresses nil)
; set addr_list(hkunz@ifi.uzh.ch) {/home/az/Mail/lists/sparclinux/106 \
; {Tue, 22 Apr 2008 11:16:46 +0200} {Hanspeter Kunz} {Hanspeter Kunz \
; <hkunz@ifi.uzh.ch>}}
	  (while (re-search-forward "^set addr_list(\\(.+?\\)) {[^{]+{[^}]+} {\\([^}]+\\)}" 
				    (point-max) t)
	    (let ((addr (match-string 1)) (name (rfc2047-decode-string (match-string 2))) (full))
	      (setq full (format "%s <%s>" name addr))
	      ;; we remember: full address and dummy value 
	      ;; or partial addresses with the full as value
	      (setq exmh-addresses (cons (cons full 1) 
					 (cons (cons addr full)
					       exmh-addresses)))
	      )
	    )
	  (kill-buffer abuf)
	  (setq exmh-addresses-lastread (current-time))))
    )
  )

(defun complete-exmh-address ()
  "Minibuffer-completes an email address using the exmh address database."
  (interactive)
  (read-exmh-addresses)
  (let ((exp) (full))
    (setq exp (completing-read "Address: " exmh-addresses))
    (if (string-match "<" exp)		; is this a full address?
	(insert exp)
      (progn				; no, so look for the full one
	(setq full (cdr (assoc exp exmh-addresses)))
	(insert full)
	))))

